import random, findbyid

class NotFoundException(Exception):
    pass

def findByID(Cls):
    class FindByIDFactory(object):
        used_ids = []

        def __init__(self, *args, **kwargs):
            cls = type(self)
            self.own_instance = Cls( *args, **kwargs)

            if not hasattr(cls, "has_instantiated_before"):
                print("reset")
                cls.instances = []
                cls.has_instantiated_before = True

            self.ID = cls.__getFreeID()
            cls.instances.append(self)

        @classmethod
        def __getFreeID(cls) -> int:
            while True:
                id = random.randint(10000, 100000)
                if id not in cls.used_ids:
                    break
            cls.used_ids.append(id)
            return id

        @classmethod
        def findByID(cls, ID: int) -> object:

            for i in cls.instances:
                if i.ID == ID:
                    return i
            else:
                raise NotFoundException("Invalid ID!")

        @classmethod
        def getInstances(cls) -> list:
            return cls.instances


    return FindByIDFactory

@findByID
class Dummy():
    def __init__(self):
        # super().__init__()
        print("Moi")

a = Dummy()
b = Dummy()

print(list(map( lambda a: a.ID, Dummy.getInstances())))
print(Dummy.getInstances())

# print(a.ID, b.ID)

print(a is Dummy.findByID(a.ID))
print(b is Dummy.findByID(b.ID))



import random

class NotFoundException(Exception):
    pass

class FindByIDFactory:
    used_ids = [] # Required for the getFreeID function to work properly.

    def __init__(self, *args, **kwargs) -> None:
        cls = type(self)
        if not hasattr(cls, "has_instantiated_before"):
            cls.instances = []
            cls.has_instantiated_before = True

        self.ID = cls.__getFreeID()
        cls.instances.append(self)

    @classmethod
    def __getFreeID(cls) -> int:
        while True:
            id = random.randint(10000, 100000)
            if id not in cls.used_ids:
                break
        cls.used_ids.append(id)
        return id

    @classmethod
    def findByID(cls, ID: int) -> object:

        for i in cls.instances:
            if i.ID == ID:
                return i
        else:
            raise NotFoundException("Invalid ID!")

    @classmethod
    def getInstances(cls) -> list:
        return cls.instances



def findByID(Cls):
    class FindByIDFactory(object):
        used_ids = []

        def __init__(self, *args, **kwargs):
            cls = type(self)
            self.own_instance = Cls( *args, **kwargs)

            if not hasattr(cls, "has_instantiated_before"):
                print("reset")
                cls.instances = []
                cls.has_instantiated_before = True

            self.ID = cls.__getFreeID()
            cls.instances.append(self)

        @classmethod
        def __getFreeID(cls) -> int:
            while True:
                id = random.randint(10000, 100000)
                if id not in cls.used_ids:
                    break
            cls.used_ids.append(id)
            return id

        @classmethod
        def findByID(cls, ID: int) -> object:

            for i in cls.instances:
                if i.ID == ID:
                    return i
            else:
                raise NotFoundException("Invalid ID!")

        @classmethod
        def getInstances(cls) -> list:
            return cls.instances


    return FindByIDFactory



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="findbyid",
    version="0.0.5",
    author="Kaappo Raivio",
    author_email="kaappo.raivio@gmail.com",
    description="A package that allows any instance of a class to be referenced with its ID that is provided by this package",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/NoSocksForYou/findbyid.git",
    packages=setuptools.find_packages(),
    classifiers=(
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ),
)
